﻿namespace HeuristicLab.Problems.ProgramSynthesis.Base.Erc.Integer {
  using HeuristicLab.Common;
  using HeuristicLab.Core;
  using HeuristicLab.Data;
  using HeuristicLab.Parameters;
  using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

  [StorableClass]
  public class IntegerRangeErcValue : WeightedErcValueItem<int> {
    private const string RangeParameterName = "Range";

    public IntegerRangeErcValue() : this(false, 0, 0) { }

    public IntegerRangeErcValue(int start, int end) : this(true, start, end) { }

    public IntegerRangeErcValue(bool isEnabled, int start, int end, double weight = 1d) : base(isEnabled, weight) {
      Name = "Integer range";
      Parameters.Add(new FixedValueParameter<IntRange>(RangeParameterName, new IntRange(start, end)));
    }

    [StorableConstructor]
    protected IntegerRangeErcValue(bool deseralizing) : base(deseralizing) { }

    public IntegerRangeErcValue(IntegerRangeErcValue origin, Cloner cloner) : base(origin, cloner) { }

    public IValueParameter<IntRange> RangeParameter
    {
      get { return (IValueParameter<IntRange>)Parameters[RangeParameterName]; }
    }

    public int Start
    {
      get { return RangeParameter.Value.Start; }
      set { RangeParameter.Value.Start = value; }
    }

    public int End
    {
      get { return RangeParameter.Value.End; }
      set { RangeParameter.Value.End = value; }
    }

    public override int GetErcValue(IRandom random) {

      return Start < End ? random.Next(Start, End) : 0;
    }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new IntegerRangeErcValue(this, cloner);
    }
  }
}
